﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

public class LevelLoader : MonoBehaviour
{
    private static LevelLoader _levelInstance;

    public static LevelLoader LevelInstance
    {
        get
        {
            return _levelInstance;
        }
    }

    private void Awake()
    {
        if(_levelInstance != null)
        {
            Destroy(gameObject);
        }
        else
        {
            _levelInstance = this;
        }
    }

    private void OnDestroy()
    {
        if(_levelInstance == this)
        {
            _levelInstance = null;
        }
    }

    private void LoadLevel(string levelName)
    {
        Scene nextScene = SceneManager.GetSceneByName(levelName);

        if(nextScene.IsValid())
        {
            SceneManager.LoadScene(levelName);
        }
        else
        {
            Debug.LogWarning("GameManager LoadLevel Error: Invalid level specified.");
        }
    }

    public void LoadNextLevel()
    {
        //int nextSceneIndex = (SceneManager.GetActiveScene().buildIndex + 1)
        //    % SceneManager.sceneCountInBuildSettings; // Cache for readibility.
        //LoadLevel(nextSceneIndex);

        SceneManager.LoadScene(SceneManager.GetActiveScene().buildIndex + 1);
    }

    public void ReloadCurrentLevel()
    {
        SceneManager.LoadScene(SceneManager.GetActiveScene().name);
    }
}
