﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityStandardAssets.CrossPlatformInput;
using UnityEngine.SceneManagement;
using LevelManagement;

public class Player : MonoBehaviour
{
    // Configuration
    [SerializeField] float exSpeed = 5f;
	[SerializeField] float jumpSpeed = 5f;
	[SerializeField] float climbSpeed = 5f;
	[SerializeField] Vector2 deathkick = new Vector2(25f,25f);
    [SerializeField] float LevelLoadDelay = 2f;

    //state
    bool isAlive = true;
    bool isControlled = true;
    bool keyHasBeenCollected = false;
    
	// Cached
    Rigidbody2D myRigidbody;
    Animator myAnimator;
    BoxCollider2D myBodyCollider;
	Collider2D myCollider2D;
    float gravityScaleAtStart;

	// Use this for initialization
	void Start ()
    {
        myRigidbody = GetComponent<Rigidbody2D>();
        myAnimator = GetComponent<Animator>();
		myBodyCollider = GetComponent<BoxCollider2D>();
		myCollider2D = GetComponent<Collider2D>();
		gravityScaleAtStart = myRigidbody.gravityScale;
    }
	
	// Update is called once per frame
	void Update ()
    {
		if (!isAlive || !isControlled) {return; }

        Run();
		Jump();
		ClimbLadder();
		FlipSprite();
		Death();
    }

    private void Run()
    {
        float controlThrow = CrossPlatformInputManager.GetAxis("Horizontal"); // -1 to 1.
        Vector2 playerVelocity = new Vector2(controlThrow * exSpeed, myRigidbody.velocity.y);
        myRigidbody.velocity = playerVelocity;

        bool playerHasHorizontalMovement = Mathf.Abs(myRigidbody.velocity.x) > Mathf.Epsilon;
        myAnimator.SetBool("Walking", playerHasHorizontalMovement);
    }

	private void Jump()
	{
		if(!myCollider2D.IsTouchingLayers (LayerMask.GetMask ("Ground"))) 
		{
			myAnimator.SetBool ("Jumping", false);
			return;
        } 
		 
		if(CrossPlatformInputManager.GetButtonDown("Jump"))
		{
			Vector2 jumpVelocityToAdd = new Vector2 (0f, jumpSpeed);
			myRigidbody.velocity += jumpVelocityToAdd;
			myAnimator.SetBool ("Jumping", true);
		}
	}


	private void ClimbLadder()
	{
		if (!myCollider2D.IsTouchingLayers (LayerMask.GetMask ("Climbing")))		
		{
			myAnimator.SetBool ("Climbing", false);
			myRigidbody.gravityScale = gravityScaleAtStart;
			return;		
		}

		float controlThrow = CrossPlatformInputManager.GetAxis ("Vertical");
		Vector2 climbVelocity = new Vector2 (myRigidbody.velocity.x, controlThrow * climbSpeed);
		myRigidbody.velocity = climbVelocity;
		myRigidbody.gravityScale = 0f;

		bool playerHasVerticalSpeed = Mathf.Abs (myRigidbody.velocity.y) > Mathf.Epsilon;
		myAnimator.SetBool ("Climbing", playerHasVerticalSpeed); // REVISAR SI ALGO PASA CON LA ANIMACION.
	}

	private void Death()
	{
		if (myBodyCollider.IsTouchingLayers (LayerMask.GetMask ("Enemy", "Hazards")))
		{
			isAlive = false;
			myAnimator.SetTrigger("Death");
			GetComponent<Rigidbody2D>().velocity = deathkick;
            StartCoroutine("RestartLevel");
		}
			
	}

    private void FlipSprite()
    {
        bool playerHasHorizontalMovement = Mathf.Abs(myRigidbody.velocity.x) > Mathf.Epsilon;

        if(playerHasHorizontalMovement)
        {
           transform.localScale = new Vector2(Mathf.Sign(myRigidbody.velocity.x), 1f);
        }
    }

    IEnumerator RestartLevel()
    {
        yield return new WaitForSeconds(3f);
        LevelLoader.LevelInstance.ReloadCurrentLevel();
    }

	void OnCollisionEnter2D(Collision2D other)
	{
		if (other.transform.tag == "MovingPlatform") 
		{		
			transform.parent = other.transform;		
		}

        if (other.transform.tag == "Key")
        {
            keyHasBeenCollected = true;
            Destroy(other.gameObject);
        }
    }

    private void OnTriggerEnter2D(Collider2D other)
    {
        if (other.transform.tag == "House" && keyHasBeenCollected)
        {
            isControlled = false;
            myAnimator.SetTrigger("Victory");
            StartCoroutine(LoadNextLevel());
        }
    }

    void OnCollisionExit2D(Collision2D other)
	{
		if (other.transform.tag == "MovingPlatform") 
		{

			transform.parent = null;

		}
	}

    IEnumerator LoadNextLevel()
    {
        yield return new WaitForSecondsRealtime(LevelLoadDelay);
        LevelLoader.LevelInstance.LoadNextLevel();
    }
}
